unit UFileIO;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs,
  UEELSBoson;

type
  Tdoublearray = array of double;
  Tintegerarray = array of integer;
  Tstringarray = array of string;
  Toscarray = array of Fosc_array;

procedure read_eelsin(
            var ImpactEnergy: double;
            var IncidenceAngle: double;
            var AngularAperturePhiA: double;
            var AngularAperturePhiB: double;
            var IntervalWMin: double;
            var IntervalWMax: double;
            var IntervalWDelta: double;
            var Comment1: string;
            var Comment2: string;
            var NumberOfLayers:   integer;
            var NumberOfEpsilons: integer;
            var NumberOfPeriodic: integer;
            var LayerName:  Tstringarray;
            var Thickness:  Tdoublearray;
            var Epsilon:    Tdoublearray;
            var NumberOfOscillators: Tintegerarray;
            var Oscillator: Toscarray
          );
procedure write_eelsin(
            const ImpactEnergy: double;
            const IncidenceAngle: double;
            const AngularAperturePhiA: double;
            const AngularAperturePhiB: double;
            const IntervalWMin: double;
            const IntervalWMax: double;
            const IntervalWDelta: double;
            const Comment1: string;
            const Comment2: string;
            const NumberOfLayers:   integer;
            const NumberOfEpsilons: integer;
            const NumberOfPeriodic: integer;
            const LayerName:  Tstringarray;
            const Thickness:  Tdoublearray;
            const Epsilon:    Tdoublearray;
            const NumberOfOscillators: Tintegerarray;
            const Oscillator: Toscarray
          );
procedure read_bosin(
            var TargetTemperature: double;
            var InstrumentalWidth: double;
            var GaussianFraction: double;
            var Asymmetry: double;
            var EMin: double;
            var EMax: double
          );
procedure write_bosin(
            const TargetTemperature: double;
            const InstrumentalWidth: double;
            const GaussianFraction: double;
            const Asymmetry: double;
            const EMin: double;
            const EMax: double
          );

implementation

procedure read_eelsin(
            var ImpactEnergy: double;
            var IncidenceAngle: double;
            var AngularAperturePhiA: double;
            var AngularAperturePhiB: double;
            var IntervalWMin: double;
            var IntervalWMax: double;
            var IntervalWDelta: double;
            var Comment1: string;
            var Comment2: string;
            var NumberOfLayers:   integer;
            var NumberOfEpsilons: integer;
            var NumberOfPeriodic: integer;
            var LayerName:  Tstringarray;
            var Thickness:  Tdoublearray;
            var Epsilon:    Tdoublearray;
            var NumberOfOscillators: Tintegerarray;
            var Oscillator: Toscarray
          );
var
  eelsin: text;
  i, j, index, Oscillator_length, here: integer;
  FileOpenDialog: TOpenDialog;
  line: string;
begin
  FileOpenDialog := TOpenDialog.Create(nil);
  FileOpenDialog.Options := [ofFileMustExist];
  FileOpenDialog.Title := 'Select the file eelsin:';
  if not FileOpenDialog.Execute then
    ShowMessage('Open the file eelsin was cancelled')
  else
  begin
  assign(eelsin, FileOpenDialog.FileName);
  reset (eelsin);
  readln(eelsin, ImpactEnergy);
  readln(eelsin, IncidenceAngle);
  readln(eelsin, AngularAperturePhiA);
  readln(eelsin, AngularAperturePhiB);
  readln(eelsin, IntervalWMin);
  readln(eelsin, IntervalWMax);
  readln(eelsin, IntervalWDelta);
  readln(eelsin, Comment1);
  readln(eelsin, Comment2);
  readln(eelsin, NumberOfLayers, NumberOfPeriodic);
  setlength(LayerName, NumberOfLayers);
  setlength(Thickness, NumberOfLayers);
  setlength(Epsilon, NumberOfLayers);
  setlength(NumberOfOscillators, NumberOfLayers);
  index := 0;
  Oscillator_length := 0;
  for i := 0 to NumberOfLayers - 1 do
  begin
    readln(eelsin, line);
    LayerName[i] := copy(line,0,10);
    line := copy(line,10, 20);
    // replace D for double precision with E
    here := pos('D', line);
    if here <> 0 then
      line[here] := 'E';
    TryStrToFloat(line, Thickness[i]);
    readln(eelsin, Epsilon[i], NumberOfOscillators[i]);
    Oscillator_length := Oscillator_length + NumberOfOscillators[i];
    setlength(Oscillator, Oscillator_length);
    for j := 1 to NumberOfOscillators[i] do
    begin
      readln(eelsin, Oscillator[index][1], Oscillator[index][2], Oscillator[index][3]);
      inc(index);
    end;
  end;
  close(eelsin);
  end;
  FileOpenDialog.Destroy;
end;

procedure write_eelsin(
            const ImpactEnergy: double;
            const IncidenceAngle: double;
            const AngularAperturePhiA: double;
            const AngularAperturePhiB: double;
            const IntervalWMin: double;
            const IntervalWMax: double;
            const IntervalWDelta: double;
            const Comment1: string;
            const Comment2: string;
            const NumberOfLayers:   integer;
            const NumberOfEpsilons: integer;
            const NumberOfPeriodic: integer;
            const LayerName:  Tstringarray;
            const Thickness:  Tdoublearray;
            const Epsilon:    Tdoublearray;
            const NumberOfOscillators: Tintegerarray;
            const Oscillator: Toscarray
          );
var
  eelsin: text;
  i, j, index, Oscillator_length, here: integer;
  FileSaveDialog: TSaveDialog;
begin
  FileSaveDialog := TSaveDialog.Create(nil);
  FileSaveDialog.Filename := 'eelsin';
  FileSaveDialog.Title := 'Save the file eelsin as:';
  if not FileSaveDialog.Execute then
    ShowMessage('Save the file eelsin was cancelled')
  else
  begin
    assign(eelsin, FileSaveDialog.FileName);
    rewrite(eelsin);
    writeln(eelsin, ImpactEnergy:10:2);
    writeln(eelsin, IncidenceAngle:10:2);
    writeln(eelsin, AngularAperturePhiA:10:2);
    writeln(eelsin, AngularAperturePhiB:10:2);
    writeln(eelsin, IntervalWMin:10:2);
    writeln(eelsin, IntervalWMax:10:2);
    writeln(eelsin, IntervalWDelta:10:2);
    writeln(eelsin, Comment1);
    writeln(eelsin, Comment2);
    writeln(eelsin, NumberOfLayers:10, NumberOfPeriodic:10);
    index := 0;
    for i := 0 to NumberOfLayers - 1 do
    begin
      writeln(eelsin, Layername[i], thickness[i]:10:2);
      writeln(eelsin, Epsilon[i]:10:2, NumberOfOscillators[i]:10);
      for j := 1 to NumberOfOscillators[i] do
      begin
        writeln(eelsin, Oscillator[index][1]:10:2, Oscillator[index][2]:10:3, Oscillator[index][3]:10:3);
        inc(index);
      end;
    end;
    close(eelsin);
  end;
  FileSaveDialog.Destroy;
end;

procedure read_bosin(
            var TargetTemperature: double;
            var InstrumentalWidth: double;
            var GaussianFraction: double;
            var Asymmetry: double;
            var EMin: double;
            var EMax: double
          );
var
  bosin: text;
  FileOpenDialog: TOpenDialog;
begin
  FileOpenDialog := TOpenDialog.Create(nil);
  FileOpenDialog.Options := [ofFileMustExist];
  FileOpenDialog.Title := 'Select the file bosin:';
  if not FileOpenDialog.Execute then
    ShowMessage('Open the file bosin was cancelled')
  else
  begin
    assign(bosin, FileOpenDialog.FileName);
    reset (bosin);
    readln(bosin, TargetTemperature);
    readln(bosin, InstrumentalWidth);
    readln(bosin, GaussianFraction);
    readln(bosin, Asymmetry);
    readln(bosin, EMin);
    readln(bosin, EMax);
    close(bosin);
  end;
  FileOpenDialog.Destroy;
end;

procedure write_bosin(
            const TargetTemperature: double;
            const InstrumentalWidth: double;
            const GaussianFraction: double;
            const Asymmetry: double;
            const EMin: double;
            const EMax: double
          );
var
  bosin: text;
  FileSaveDialog: TSaveDialog;
begin
  FileSaveDialog := TSaveDialog.Create(nil);
  FileSaveDialog.Filename := 'bosin';
  FileSaveDialog.Title := 'Save the file bosin as:';
  if not FileSaveDialog.Execute then
    ShowMessage('Save the file bosin was cancelled')
  else
  begin
    assign(bosin, FileSaveDialog.FileName);
    rewrite(bosin);
    writeln(bosin, TargetTemperature:8:2, '    Target Temperature');
    writeln(bosin, InstrumentalWidth:8:2, '    Instrumental Width');
    writeln(bosin, GaussianFraction:8:2,  '    Gaussian Fraction');
    writeln(bosin, Asymmetry:8:2,         '    Asymmetry');
    writeln(bosin, EMin:8:2,              '    Minimum Energy');
    writeln(bosin, EMax:8:2,              '    Maximum Energy');
    close(bosin);
  end;
  FileSaveDialog.Destroy;
end;

end.

