module seteps_mod
contains
subroutine seteps(nos, osc_size, osc, epsinf, wn, nLayer, eps)

  ! ******************************************************************
  ! * set up long-wavelength dielectric functions of the layers for  *
  ! * the present frequency wn (in cm**-1)                           *
  ! ******************************************************************
  
  implicit none

  integer, intent(in) :: nLayer
  integer, intent(in) :: nos(:)
  integer, intent(in) :: osc_size
  double precision, intent(in) :: osc(:,:)
  !f2py depend(osc_size) osc
  double precision, dimension(nLayer),intent(in) :: epsinf
  double precision,  intent(in) :: wn
  double complex, dimension(nLayer), intent(out) :: eps
  !f2py depend(nLayer) nos, epsinf, eps
  
  double complex :: nomi, deno, addeps, addlorentz
  double precision :: wn2, b, gamma
  integer j, k, l, m
  logical debugFirstRun

  common /control/ debugFirstRun
   
  j = 0
  do l = 1, nLayer      ! loop over different thin film layers
    ! The numbering does not work with a mix of Lorentz and Kurosawa/extended Drude terms
    ! Either Lorentz with no Drude or Kurosawa and extended Drude terms
    m = nos(l)/2      ! m number of TO modes = offset to reach the LO mode in the joint TO-LO list
    if (osc(1,1) < 0.0) then
      m = nos(l)
    endif
    nomi = dcmplx(1.0d0, 0.0d0)
    deno = dcmplx(1.0d0, 0.0d0)
    addeps = dcmplx(0.0d0, 0.0d0)
    addlorentz = dcmplx(0.0d0, 0.0d0)
    wn2 = wn**2
    do k = 1, m     ! loop over all TO modes
      j = j + 1

      if (osc(1,j) > 0.0) then     ! positive TO mode: 'Kurosawa' form: _Multiplicative_ phonon mode
        b = wn/osc(1, j+m)
        nomi = nomi * osc(1, j+m)**2 * (1.0 - b * dcmplx(b, osc(3,j+m)/osc(1, j+m)) )
        deno =deno * (osc(1,j)**2 - wn * dcmplx( wn, osc(3,j) ) )
        j = j + m     ! we have already looped over the LO modes, therefore increase the index
      else if (osc(1,j) < 0.0) then! Negative TO mode means: _Additive_ Lorentz oscillator with Q
        gamma = abs(osc(1,j)) * osc(3,j)                                                          ! osc(3,j) is lambda. gamma = omega_TO * lambda
        addlorentz = addlorentz + osc(1,j)**2 * osc(2,j) /dcmplx(osc(1,j)**2 - wn2, -wn * gamma)  ! Sign of imaginary part changed (WFW)
      
      else                      ! osc(1,j) = 0   -> it is a double damping extended Drude term
        addeps = addeps - dcmplx(osc(1,j+m)**2, wn*(osc(3,j)-osc(3,j+m))) /dcmplx(wn2, wn*osc(3,j))
        j = j + m     ! we have already looped over the LO modes, therefore increase the index
      end if

    enddo
    eps(l) = epsinf(l) * (nomi / deno + addeps) + addlorentz ! brackets changed by HHe 230915
  enddo
  debugFirstRun = .false.
  return
end subroutine seteps
end module seteps_mod
