program eels

! ******************************************************************
! *                                                                *
! * compute the classical eels spectrum of an arbitrary plane-     *
! * statified medium made from isotropic materials in specular     *
! * geometry using the dielectric theory of eels.                  *
! *                                                                *
! ******************************************************************

  use doeels_mod
  use oscillator_mod

  implicit none

  double precision :: e0, theta, phia, phib, wmin, wmax, dw
  integer :: i, j, jos, k, l, layers, neps, nper, nw
  logical :: user
  character (len = 72) :: comment(2)
  character (len = 10) :: contrl

  double precision, allocatable :: thick(:), epsinf(:), osc(:, :)
  integer, allocatable :: nos(:), oscType(:), tmp_oscType(:)
  character (len = 10), allocatable :: name(:)
  double precision, allocatable :: wn_array(:), f(:)

  integer :: old_size, old_size_1, old_size_2
  double precision, allocatable :: tmp_osc(:, :)
  logical :: debug
  integer :: ioStatus

!  integer, parameter :: name_length = 10
! *** read spectrometer parameters

  call change_working_dir()
  open(unit = 11, file = 'eelsin')
! impact energy (eV)
  read(11, *) e0
! incidence angle (°)
  read(11, *) theta
! angular apertures of the elliptic detector (°)
  read(11, *) phia
  read(11, *) phib
! energy-loss interval and step size (cm**-1)
  read(11, *) wmin
  read(11, *) wmax
  read(11, *) dw
! comment lines
  read(11, '(a72)') (comment(k), k = 1, 2)

  write(*,*) 'program eels (September 2020)'
  write(*,'(a, f6.2, a, f5.1, a, f5.2, a, f5.2, a)') &
    ' e0 = ', e0, ' eV, theta = ', theta, '°, phia = ', &
    phia, '°, phib = ', phib, '°'
  write(*,'(a, g11.4, a, g11.4, a, g11.4, a)') &
    ' energy losses from', wmin, ' to', wmax, ', step = ', dw, ' cm**-1'
  write(*,*) comment(1)
  write(*,*) comment(2)

  if (phia <= 0.0d0 .or. phib <= 0.0d0) then
    stop '*** wrong input ***'
  endif
  if (e0 <= 0.0d0 .or. theta + phia >= 90.0d0) then
    stop '*** bad input ***'
  endif

! *** read target specifications

  read(11, *) layers, nper
  user = layers == 0
  if (.not. user) then
    neps = layers
    if (nper == -1) then
      neps = layers + 1
      nper = 1
      write(*,*) 'The substrate is an anisotropic uniaxial material'
    endif
    if (layers < 0 .or. nper < 1 .or. nper > layers) then
      stop '*** invalid target specifications ***'
    endif
    write(6, 101) layers, nper
    jos = 0
    allocate (name(neps), thick(neps), epsinf(neps), nos(neps))
    do l = 1, neps
      if (l <= layers) then
        read(11, 102) name(l), thick(l)
      endif
      read(11, *) epsinf(l), nos(l)
      write(6, 103)
      if (nos(l) <= 0) then
        if (l <= layers) then
          write(6, 104) l, name(l), thick(l), epsinf(l)
        endif
        if (l  > layers) then
          write(6, 105) epsinf(l)
        endif
      else
        do j = 1, nos(l)
          if (.not. allocated(osc)) then
            allocate(osc(4, nos(l)))
          else if (j == 1) then
 !           call extend3(osc, nos(j))
            old_size_1 = size(osc, 1)
            old_size_2 = size(osc, 2)
            allocate(tmp_osc(old_size_1, old_size_2 + nos(l)))
            tmp_osc(1:old_size_1, 1:old_size_2) = osc
            deallocate(osc)
            call move_alloc(tmp_osc, osc)
          endif
          if (.not. allocated(oscType)) then
            allocate(oscType(nos(l)))
          else if (j == 1) then
            old_size = size(oscType)
            allocate(tmp_oscType(old_size + nos(l)))
            tmp_oscType(1:old_size) = oscType
            deallocate(oscType)
            call move_alloc(tmp_oscType, oscType)
          endif
          jos = jos + 1
          read(11, *) (osc(k, jos), k = 1, 3)
          oscType(jos) = Lorentz
          if (osc(2, jos) == -1.0) then  ! Drude term
            osc(2, jos) = osc(3, jos)
            osc(3, jos) = 0.0  
            oscType(jos) = Drude
          endif
          if ((j == nos(l) / 2 + 1) .and. (nos(l) > 1)) then
            write(6, 106)
            write(6, 107)
          endif
          if (j == 1) then
            if (l <= layers) then
              write(6, 104) l, name(l), thick(l), epsinf(l), (osc(i, jos), i = 1, 4)
            else
              write(6, 105) epsinf(l), (osc(i, jos), i = 1, 4)
            endif
          else
            write(6, 108) (osc(i, jos), i = 1, 4)
          endif
        enddo
      endif
    enddo
    write(*,*)
    read(11, 102, IOSTAT = ioStatus) contrl
    if (ioStatus /= 0) then
      contrl = ''
    endif
  endif

  close (unit = 11)

  nw = 1 + int((wmax - wmin) / dw)
  allocate (wn_array(nw), f(nw))

  debug = .false.
  call doeels(e0, theta, phia, phib, wmin, wmax, dw, comment, size(comment),   &
              layers, neps, nper, name, size(name), thick, epsinf, nos, oscType, osc, size(osc, 2),&
              contrl, wn_array, debug, f, size(wn_array))

  open (unit = 12, file = 'eelsou')
  write (12, 207) e0, theta, phia, phib
  write (12, '(a72)') comment(1)
  do i = 1, nw
    write (12, '(2e15.7)') wn_array(i), f(i)
  enddo
  close (unit = 12)

  stop

101 format(i3, ' layer(s), nper = ', i2//'   l', 2x, 'material', 7x,  &
      'thickness', 5x, 'epsinf', 4x, 'wto , wp', 5x, 'q', 7x, 'gam/wto')
102 format(a10, d15.5)
103 format(1x, 72('-'))
104 format(1x, i3, 2x, a10, g15.3, f10.4, f12.4, f10.4, f9.4, f9.4)
105 format(31x, f10.4, f12.4, f10.4, f9.4, f9.4)
106 format(45x, 'wlo , wp', 5x, 'q', 7x, 'gam/wlo')
107 format(45x, 28('-'))
108 format(41x, f12.4, f10.4, f9.4, f9.4)
207 format('e0 =', f6.2, ' theta =', f5.1, ' phia =', f5.2, ' phib =', f5.2)
end program eels
