module fint3_mod
contains
double precision function fint3(u, eps, thick, layers, nper, eps_size)

! ******************************************************************
! *                                                                *
! * integration over the azimutal angle from phi1 > 0 to phi2 < pi *
! *                                                                *
! * eps:      epsilon                                              *
! * thick:    thickness                                            *
! * layers:   number of layers                                     *
! * nper:     number of periodic layers                            *
! * eps_size: number of layers                                     *
! *                                                                *
! ******************************************************************

  use surlos_mod
  use param_mod

  implicit none

  double precision, intent(in) :: u
  double complex, intent(in) :: eps(eps_size)
  double precision, intent(in) :: thick(eps_size)
  integer, intent(in) :: layers, nper, eps_size

  double precision :: arg, phi1, phi2, rac, t
  double precision :: phint, usurlo

!  write (*,*) 'fint3:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  if (u == 0.0d0) then
    fint3 = 0.0d0
    return
  endif
  rac = dsign(1.0d0, acoef) * cospsi * dsqrt((1.0d0 - elleps) * acoef * (um - u) * (um + u))
  arg = (bcoef - rac) / (u * acoef)
  if (dabs(arg) > 1.0d0) then
    arg = dsign(1.0d0, arg)
  endif
  phi2 = dacos(arg)
  fint3 = phint(phi2, tanpsi, u)
  arg = (bcoef + rac) / (u * acoef)
  if (dabs(arg) > 1.0d0) then
    arg = dsign(1.0d0, arg)
  endif
  phi1 = dacos(arg)
  fint3 = fint3 - phint(phi1, tanpsi, u)
  if (rational) then
    return
  endif
  if (user) then
    fint3 = fint3 * usurlo(ru * u, wn)
  else
    fint3 = fint3 * surlos(ru * u, eps, thick, layers, nper)
    if (dlimf > 0.0d0) then
      t = ru * u * dlimf
      fint3 = fint3 * (1.d0 + t * dlog(t / (t + 0.26d0)))**2 / (1.d0 + 1.40d0 * t)
    endif
  endif
  return
end function fint3
end module fint3_mod
