module getOscillator_mod
  contains
  subroutine getOscillator(termListItem,  oscType, osc, jos)

    use, intrinsic :: iso_fortran_env, only:  output_unit
    use oscillatorType_mod
    use fortran_yaml_c

    class(type_list_item), pointer, intent(in) :: termListItem
    integer, allocatable, intent(in out) :: oscType(:)
    double precision, allocatable, intent(in out) :: osc(:, :)
    integer, intent(in) :: jos

    type(type_error), allocatable :: io_err
    character(:), allocatable :: oscTypeString

    write (*,*) 'Oscillator term', jos, ':'
    call termListItem%node%dump(unit = output_unit, indent = 0)

    select type (oscillatorTerm => termListItem%node)
    class is (type_dictionary)
      oscTypeString = oscillatorTerm%get_string('Type', error = io_err)
      if (allocated(io_err)) then
        print *, io_err%message
        stop 1
      endif
      oscType(jos) = convert(oscTypeString)
      select case (oscType(jos))
      case (Lorentz)
        osc(1,jos) = oscillatorTerm%get_real('wTO', error = io_err)
        osc(2,jos) = oscillatorTerm%get_real('Qk', error = io_err)
        osc(3,jos) = oscillatorTerm%get_real('yk', error = io_err)
        osc(4,jos) = 0.0
      case (Kurosawa)
        osc(1,jos) = oscillatorTerm%get_real('wTO', error = io_err)
        osc(2,jos) = oscillatorTerm%get_real('yTO', error = io_err)
        osc(3,jos) = oscillatorTerm%get_real('wLO', error = io_err)
        osc(4,jos) = oscillatorTerm%get_real('yLO', error = io_err)
      case (Drude)
        osc(1,jos) = oscillatorTerm%get_real('wP', error = io_err)
        osc(2,jos) = oscillatorTerm%get_real('y', error = io_err)
        osc(3,jos) = 0.0
        osc(4,jos) = 0.0
      case (extendedDrude)
        osc(1,jos) = oscillatorTerm%get_real('wP', error = io_err)
        osc(2,jos) = oscillatorTerm%get_real('yp', error = io_err)
        osc(3,jos) = oscillatorTerm%get_real('yP', error = io_err)
        osc(4,jos) = 0.0
      end select
    end select

    write (*,*) 'Oscillator: ', oscTypeString, osc(1,jos), osc(2,jos), osc(3,jos), osc(4,jos)
    return

  end subroutine getOscillator
end module getOscillator_mod
