module get_commandline_options_mod
contains
subroutine get_commandline_options(eelsin_name, bosonout_name)

! This routine defines the commandline options, parses the commandline and sets the 
! filenames of I/O files.
!
! It uses sufr_getopt from libSUFR and has been derived from getopt_long_example.f90
!
! KMS

  use sufr_getopt, only: getopt_t, getopt_long, longOption, optArg, getopt_long_help

  implicit none
  
  character (len = :), allocatable, intent(in out) :: eelsin_name, bosonout_name
    
  include 'version.inc'

  character :: option
  integer :: status

  ! Set up the longOpts struct to define the valid options:
  ! short option, long option, argument (no = 0 / yes = 1), short description
  type(getopt_t) :: longOpts(6) =                                                &
      [                                                                          &
       getopt_t('v', 'version',   0, 'Print version'),                           &
       getopt_t('V', 'version',   0, 'Print version'),                           &
       getopt_t('h', 'help',      0, 'Print options'),                           &
       getopt_t('d', 'dir',       1, 'Change I/O directory'),                    &
       getopt_t('e', 'eelsin',    1, 'Name of EELS input file (eelsin.yaml)'),   &
       getopt_t('b', 'bosonout',  1, 'Name of BOSON output file (bosonout.txt)') &
      ]

  do  ! scan all the command-line parameters
     
     ! getopt_long() returns a single character" ">","!",".", or the short-option character (e.g. "a" for -a).
     !   It also sets two 'global' variables through the SUFR_getopt module:
     !   - longOption:  the full option (e.g. "-a" or "--all") including the dashes
     !   - optArg:      the argument following the option (if required and present)
     option = getopt_long(longOpts)
     
     ! Do different things depending on the option returned:
     select case(option)
     case('d')  ! Change I/O directory
        status = chdir(trim(optArg))
        if (status /= 0) then
          write (*,*) 'WARNING: change directory failed!'
          write (*,*) 'Directory tried: ', trim(optarg)
          write (*,*) 'Error code (see: man chdir): ', status
          write (*,*) 'Continuing in the start directory.'
          write (*,*) ''
        end if
     case('e')
        eelsin_name = optArg
     case('b')
        bosonout_name = optArg
     case('v')
        write (*,*) 'eels-boson version: ' // version
        stop
     case('V')
        write (*,*) 'eels-boson version: ' // version
        stop
     case('h')
        call getopt_long_help(longOpts)
        stop
     case('!')  ! Unknown option (starting with "-" or "--")
        write (*,*) 'WARNING: unknown option: ' // trim(optArg)
        call getopt_long_help(longOpts)
        stop
     case('.')  ! Parameter is not an option (i.e., it doesn't start with "-" or "--")
        write (*,*) 'WARNING: parameter without option: ' // trim(optArg)
        call getopt_long_help(longOpts)
        stop
     case default
        write (*,*) 'WARNING: unknown option: ' // trim(longOption)
        call getopt_long_help(longOpts)
        stop
     case('>')  ! Last parameter. Exit case statement
        exit
     end select
  end do
  
  return
end subroutine get_commandline_options
end module get_commandline_options_mod
